<?php
/*
 *      xml.php is a part of GCweb (unofficial web interface for gcstar)
 *
 *      Copyright 2007 Jonas Fourquier <http://jonas.tuxfamily.org>
 *
 *      GCstarWeb is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

if (strtoupper(substr(php_uname(), 0, 3)) === 'WIN') {
    function strptime($date,$format='') {
        /*
         * strptime n'existe pas sous windows le tri pas date est donc impossible.
         * Substitution de la fonction dispo sous GNU/Linux par une fonction retournant
         * tjs faux afin de GCweb fonctionne quand meme bien que partiellment
         */
        return False;
    }
}

class bdd
{
    function xml2array($xmlfile)
    {
        /*
         * Charge le fichier xml dans un tableau
         */

        global $collec, $conf;

        $cacheFullBDD = PATH_GCWEB.'/cache/bdd/'.$collec['id'].'_fullBDD.gz';
        if (!file_exists($cacheFullBDD))
            $cacheFullBDD_mtime = 0;
        else
            $cacheFullBDD_mtime = filemtime($cacheFullBDD);

        if (  $conf['noCacheBDD']
            || isset($_GET['purge'])
            || $cacheFullBDD_mtime < filemtime($xmlfile)
        ) {
            // On charge le fichier xml pour créer le cache
            if (!$xml = simplexml_load_file($xmlfile))
                return sprintf(__('erreur : impossible de charger le xml %s.'),$xmlfile);


            // Parcourir les items
            $id = 0;
            $offset4duplicateID = 0;
            $bdd = array();
            foreach($xml->item as $itemxml) {
                $id ++;
                $item = array();

                //A partire de GCweb 1.3 certaine donnée sont enregistrer dans le atribut de item
                foreach ($itemxml->attributes() as $champ => $value) {
                    if ($value != "")
                        $item[$champ] = convstr::xml2motor(nl2br(htmlspecialchars(trim($value))));
                    else
                        $item[$champ] = $conf['champVide'];
                }

                //Parcourir les champs de l'item
                foreach ($itemxml->children() as $childxml) {

                    $champ = $childxml->getName();
                    //echo " || $champ = $childxml";

                    if ($childxml->line->col) {
                        //echo "<br /> $champ : ";
                        //Si le champ est un liste (genre, auteurs ...) parcourt cette liste
                        foreach ($childxml->children() as $linexml) {
                            //Stocker le champs de type liste
                            if (count($linexml->col) == 1) {
                                if ($linexml->col != "")
                                    $item[$champ][] = convstr::xml2motor(nl2br(htmlspecialchars(trim($linexml->col))));
                                else
                                    $item[$champ][] = $conf['champVide'];
                            } else {
                                $line = array();
                                foreach ($linexml->children() as $colxml) {
                                    if ($colxml != "")
                                        $line[] = convstr::xml2motor(nl2br(htmlspecialchars(trim($colxml))));
                                    else
                                        $line[] = $conf['champVide'];
                                }
                                $item[$champ][] = $line;
                            }
                        }
                    } else {
                        //Sinon stocker le champs de type str, int ...
                        //echo "<br /> $champ :" . convstr::xml2motor($childxml)."<br />";
                        if ($childxml != "")
                            $item[$champ] = convstr::xml2motor(nl2br(htmlspecialchars(trim($childxml))));
                        else
                            $item[$champ] = $conf['champVide'];
                    }
                }

                //Stocker l'item "tableau php" dans un monstre tableau contenant tout le items
                if (!isset($item['id'])) {
                    if (isset($item['gcsautoid']))
                        $item['id'] = $item['gcsautoid'];
                    else
                        $item['id'] = $id;
                }

                //Si un décalage d'id existe à cause d'du dupliqué, l'applique
                if ($offset4duplicateID != 0)
                    $item['id'] = strval($item['id'] + $offset4duplicateID);

                while (array_key_exists($item['id'],$bdd)) {
                    //Si il y a une duplication d'un id, le majorer jusqu'à trouver une place libre
                    $offset4duplicateID ++;
                    $item['id'] = strval($item['id'] + $offset4duplicateID);
                }

                include PATH_GCWEB.'/conf/plugins4item.php';

                $bdd[$item['id']] = $item;

                unset($item);
            }

            include PATH_GCWEB.'/conf/plugins4bdd.php';


            if (!$conf['noCacheBDD']) {
                $fp = gzopen($cacheFullBDD,'w');
                gzwrite($fp,(serialize($bdd)));
                gzclose($fp);
            }
        }  else {
            //lecture cache de la BDD complête non triée
            $bdd = unserialize(implode("", gzfile($cacheFullBDD)));
            //$collec['nbItemsBDD'] = count($bdd);
        }

        return $bdd;
    }


    function tri($array, $sortStr)
    {
        /*
         * Trier en tableau selon 1 à 6 clefs.
         * $array : tableau à trier
         * $sortStr : Chaine de tri
         *       ex : "serieDSC,titleASC" = tri décroissant selon les séries
         *                                  puis croissant selon les titres
         * Retourne le tableau trier
         */

        global $msg, $conf;

        //Transformation de la chaine de tri en tableau de tri
        //echo "trier selon $sortStr";
        $tmpSorts = explode(',',$sortStr);
        if (count($tmpSorts) > 6) {
            $msg .= '<p class="warning">'.__('Attention ! Il est impossible de trier un tableau
                selon plus de 6 clefs. Le tableau a été trié selon le 6 première clef').'</p>';
            array_splice ($tmpSorts, 6);
        }


        foreach ($tmpSorts as $sort) {
            if (substr($sort,-3) == 'DSC')
                $sorts[] = array(substr($sort,0,-3),SORT_DESC);
            elseif (substr($sort,-3) == 'ASC')
                $sorts[] = array(substr($sort,0,-3),SORT_ASC);
            else
                $sorts[] = array($sort,SORT_ASC);
        }


        //Pour l'utilisation de la fonction array_multisort création des tableaux
        //de tri.
        //Les valeurs (qui servent au tri) sont débarassée des majuscules et des
        //accents pour un tri insensible à ceux-ci.
        foreach ($array as $key => $row) {
            foreach ($sorts as $sort) {
                $value = $row[$sort[0]];
                if (is_array($value)) #Si le champ est de type tableau, tri selon sa 1ère clef
                    $value = $value[0];
                if ($datadate = strptime($value,$conf['dateGCstar'])) {
                    //Si GCweb juge que c'est une date, y converti en integer
                    $value = $datadate['tm_year'] * 512 + $datadate['tm_yday']; // * 512pour avoir un truc plus grand que 366 jours (année bisextile)
                } else { //Sinon ben... bête chaîne
                    $value = @strtolower($value); //suppression de majuscule
                    $value = strtr($value, //suppression de accent
                        array('À'=>'a','Á'=>'a','Â'=>'a','Ã'=>'a','Ä'=>'a','Å'=>'a','Æ'=>'a',
                            'Ç'=>'c',
                            'È'=>'e','É'=>'e','Ê'=>'e','Ë'=>'e',
                            'Ì'=>'i','Í'=>'i','Î'=>'i','Ï'=>'i',
                            'Ð'=>'d','Ñ'=>'n',
                            'Ò'=>'o','Ó'=>'o','Ô'=>'o','Õ'=>'o','Ö'=>'o','Ø'=>'o',
                            'Ù'=>'u','Ú'=>'u','Û'=>'u','Ü'=>'u',
                            'Ý'=>'y',
                            'Þ'=>'b','ß'=>'b',
                            'à'=>'a','á'=>'a','â'=>'a','ã'=>'a','ä'=>'a','å'=>'a','æ'=>'a',
                            'ç'=>'c',
                            'è'=>'e','é'=>'e','ê'=>'e','ë'=>'e',
                            'ì'=>'i','í'=>'i','î'=>'i','ï'=>'i','ð'=>'i',
                            'ñ'=>'n',
                            'ò'=>'o','ó'=>'o','ô'=>'o','õ'=>'o','ö'=>'o','ø'=>'o',
                            'ù'=>'u','ú'=>'u','û'=>'u',
                            'ý'=>'y','ý'=>'y','þ'=>'y','ÿ'=>'y',
                            'Ŕ'=>'r','ŕ'=>'r'));
                    foreach ($conf['ignoreString4sort'] as $delStr)
                        $value = str_replace(convstr::xml2motor($delStr),'',$value);
                }
                $tmparray[$sort[0]][$key] = $value;
            }
        }

        //Tri selon 1 à 6 clef (la case correspond au nombre de clef)
        switch (count($sorts))
        {
            case 1:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,$array
                );
                break;
            case 2:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,
                    $tmparray[$sorts[1][0]],$sorts[1][1],SORT_REGULAR,$array
                );
                break;
            case 3:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,
                    $tmparray[$sorts[1][0]],$sorts[1][1],SORT_REGULAR,
                    $tmparray[$sorts[2][0]],$sorts[2][1],SORT_REGULAR,$array
                );
                break;
            case 4:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,
                    $tmparray[$sorts[1][0]],$sorts[1][1],SORT_REGULAR,
                    $tmparray[$sorts[2][0]],$sorts[2][1],SORT_REGULAR,
                    $tmparray[$sorts[3][0]],$sorts[3][1],SORT_REGULAR,$array
                );
                break;
            case 5:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,
                    $tmparray[$sorts[1][0]],$sorts[1][1],SORT_REGULAR,
                    $tmparray[$sorts[2][0]],$sorts[2][1],SORT_REGULAR,
                    $tmparray[$sorts[3][0]],$sorts[3][1],SORT_REGULAR,
                    $tmparray[$sorts[4][0]],$sorts[4][1],SORT_REGULAR,$array
                );
                break;
            default:
                array_multisort(
                    $tmparray[$sorts[0][0]],$sorts[0][1],SORT_REGULAR,
                    $tmparray[$sorts[1][0]],$sorts[1][1],SORT_REGULAR,
                    $tmparray[$sorts[2][0]],$sorts[2][1],SORT_REGULAR,
                    $tmparray[$sorts[3][0]],$sorts[3][1],SORT_REGULAR,
                    $tmparray[$sorts[4][0]],$sorts[4][1],SORT_REGULAR,
                    $tmparray[$sorts[5][0]],$sorts[5][1],SORT_REGULAR,$array
                );
                break;
        }
        return $array;
    }


    function filtre($array, $filtreStr)
    {
        /*
         * Flitre un tableau selon des conditions "contien", "plus petit"
         * "égal" ... qui peuvent se compiné avec de "et" ou des "ou"
         * $array : tableau à filtrer
         * $filtreStr : chanine de filtre
         *     ex : title=Sillage,authors=Buchet
         *
         * Return le tableau filter
         */

        global $msg, $conf;

        $newarray = array();
        $conditions = explode(',',$filtreStr);

        /*?><pre><?php print_r($conditions) ?></pre><?php*/


        // Parcourt le condition
        foreach ($conditions as $condition) {
            if (strpos($condition,'!=='))
                $cond = '!=='; //Ne contien pas la chaine exact
            elseif (strpos($condition,'!='))
                $cond = '!='; //Ne contien pas
            elseif (strpos($condition,'=='))
                $cond = '=='; //Strictement =
            elseif (strpos($condition,'<='))
                $cond = '<='; //Plus petit ou =
            elseif (strpos($condition,'>='))
                $cond = '>='; //Plus grand ou =
            elseif (strpos($condition,'='))
                $cond = '='; //contient
            elseif (strpos($condition,'<'))
                $cond = '<'; //plus petit
            elseif (strpos($condition,'>'))
                $cond = '>'; //plus grand
            else {
                $msg .= '<p class="error">'.sprintf(__('Erreur de syntaxe sur la condition "%s"'),$condition).'"</p>';
                $conf['noCacheBDD'] = True; //Y'a une erreur; ne pas mettre en cache
                break;
            }

            list($key,$value) = explode ($cond,$condition);
            $key = trim($key);
            $value = trim($value);
            if ($key{0} == '|' || $key{0} == '&') //Si un signe de conditions se trouve
                $key =substr($key,1); //devant la clef, le supprime.

            if ($datadate = strptime($value,$conf['dateGCstar']))
                $value = $datadate['tm_year'] * 512 + $datadate['tm_yday'];
            if ($condition{0} == '&' && isset($newarray))
                $searchinarray = $newarray;
            else
                $searchinarray = $array;

            foreach ($searchinarray as $item) {
                // Parcourt le tableau principal et stock dans un tableur
                // temporaire les items au conditions correct
                $id = $item['id'];

                if (!isset($item[$key])) {
                    $msg .= '<p>'.sprintf(__('Le champs "%s" n\'existe pas, filtrage avec celui-ci impossible'),$key).'</p>';
                    $conf['noCacheBDD'] = True; //Y'a une erreur; ne pas mettre en cache
                    break;
                }

                //itemKews doit être un tableau 1D
                if (is_array($item[$key])) {
                    if (is_array($item[$key][0])) {
                        //Pour le support des element 2D (plusieur ligne et plusieur colonne)
                        //transformation en tableau 1D contenant toute les donnée
                        $itemKeys = array();
                        foreach ($item[$key] as $keys)
                            $itemKeys = array_merge($itemKeys,$keys);
                    } else {
                        //On balance la clef sans la transphormer
                        $itemKeys = $item[$key];
                    }
                } else {
                    //La clef n'est pas un tableaus, on met se valeur dans la 1êre case
                    $itemKeys[] = $item[$key];
                }

                $ok = False;
                $ko = False;

                foreach ($itemKeys as $itemKey) {
                    if (in_array($cond,array('<','>','<=','>=')) && !is_numeric($itemKey)) {
                        // Ce n'est pas du numérique donc certainement une date. essaie de convertir
                        // en timestamp
                        if ($datadate = strptime($itemKey,$conf['dateGCstar']))
                            $itemKey = $datadate['tm_year'] * 512 + $datadate['tm_yday'];
                    }

                    switch ($cond)
                    {
                        case '!==':
                            if ($itemKey != $value)
                                $ok = True;
                            else
                                $ko = True;
                            break;
                        case '!=':
                            if (!stristr($itemKey,$value))
                                $ok = True;
                            else
                                $ko = True;
                            break;
                        case '==':
                            if ($itemKey == $value)
                                $ok = True;
                            break;
                        case '<=':
                            if ($itemKey <= $value)
                                $ok = True;
                            break;
                        case '>=':
                            if ($itemKey >= $value)
                                $ok = True;
                            break;
                        case '=':
                            if (stristr($itemKey,$value))
                                $ok = True;
                            break;
                        case '<':
                            if ($itemKey < $value)
                                $ok = True;
                            break;
                        case '>':
                            if ($itemKey > $value)
                                $ok = True;
                            break;
                    }
                }
                if ($ok && !$ko)
                    $tmparray["$id"] = $item;
                unset($itemKeys);
            }

            switch ($condition{0})
            {
                case '&':
                    $newarray = $tmparray;
                    break;
                case '|':
                default:
                    if (isset($tmparray))
                        $newarray += $tmparray;
                    break;
            }
            unset ($tmparray);
        }
        return $newarray;
    }

    function filterSortSliceAndCache($filter='none',$sort='none',$start=False,$end=False,$noPurgeCache=False)
    {
        /*
         * Tri, filtre, extrait un portition et met en cache le resultat et le retourne
         */

        global $conf,$collec;

        //$collec = $info['collections'][$idcollec];


        $cacheFile = PATH_GCWEB.'/cache/bdd/'.$collec['id'].'_'.str_replace('/','-',$filter).'_'.$sort;

        if ($start) $cacheFile .= '_'.$start;
        else        $cacheFile .= '_0';
        if ($end) {
            $cacheFileStatic = $cacheFile.'-'.$end.'.static.gz';
            $cacheFile .= '-'.$end.'.gz';
        } else {
            $cacheFileStatic = $cacheFile.'-end.static.gz';
            $cacheFile .= '-end.gz';
        }

        if ($noPurgeCache || file_exists($cacheFileStatic))
            $cacheFile = $cacheFileStatic;

        $xmlfile = dirname(__FILE__).'/../collections/'.$collec['dir'].'/'.$collec['xml'];

        if ($conf['noCacheBDD'] || isset($_GET['purge']) || !file_exists($cacheFile)) {
            //Pas de cache spécifier en config ou purge demander via url
            $createCache = True;
        } else {
            //date des différents fichier
            $dateCache  = filemtime($cacheFile);
            $dateXml    = filemtime($xmlfile);

            if ($dateCache < $dateXml) {
                //Si la date du fichier cache est plus vielle que celle du xml
                $createCache = True;
            } else {
                $createCache = False;
            }
        }

        if ($createCache) {
            $bdd = bdd::xml2array($xmlfile);

            $collec['nbItemsBDD'] = count($bdd);

            //Application des filtres
            if ($filter != 'none')
                $bdd = bdd::filtre($bdd,$filter);
            //Tri
            if ($sort != 'none' && count($bdd) != 0)
                $bdd = bdd::tri($bdd,$sort);

            if ($start || $end) {
                if (!$start)    $start = 0;
                if ($end)       $bdd = array_slice($bdd,$start,$end);
                else            $bdd = array_slice($bdd,$start);
            }

            // ecriture du cache
            if (!$conf['noCacheBDD']) {
                $fp = gzopen($cacheFile,'w');
                gzwrite($fp,(serialize(array($collec['nbItemsBDD'],$bdd))));
                gzclose($fp);
            }
        } else {
            // lecture du cache
            $cache = unserialize(implode('', gzfile($cacheFile)));
            $collec['nbItemsBDD'] = $cache[0];
            $bdd = $cache[1];
        }

        include PATH_GCWEB.'/conf/plugins4aftercache.php';

        return $bdd;
    }
}

?>
